# Deploy a Django app from git
define django (
  $source,                     # Source of app in git
  $url,                        # URL that will be used to serve the app
  $path = "/var/www/${name}",  # Destination path on filesystem
  $ensure = 'present',
  $revision = undef,           # Revision of the app
  $ssl      = false,           # Enable SSL
  $identity = undef,           # SSH key for git repo
  $port     = undef,           # Override port
  $ssl_cert = undef,           # Path to SSL cert
  $ssl_key  = undef,           # Path to SSL key
) {

  # Directory layout for a typical django app
  # /var/www/django-app    This level stored in the git repo and defined in $path
  # ├─ requirements.txt    Lists dependencies
  # ├─ virtualenv          Listed in .gitignore, generated by virtualenv
  # ├─ setup.py
  # └─ project-name        A collection of applications
  #    ├─ application1     One application per "feature"
  #    ├─ application2
  #    └─ project-name
  #       ├─ wsgi.py       Called by apache mod_wsgi
  #       └─ urls.py       Maps URLs onto applications

  # Initialise Apache and modules
  include apache
  include apache::mod::wsgi

  # Install & configure Python
  class { 'python' :
    virtualenv => true,
    dev        => true,
  }

  # Create the directory where the app will be installed
  file { $path:
    ensure => directory,
  }

  # Check out the app from version control
  vcsrepo { $name:
    ensure   => present,
    path     => $path,
    provider => 'git',
    source   => $source,
    revision => $revision,
    identity => $identity,
    require  => File[$path],
  }

  # Create a virtualenv and install deps
  python::virtualenv { $name:
    ensure       => present,
    venv_dir     => "${path}/virtualenv",
    requirements => "${path}/requirements.txt",
    require      => Vcsrepo[$path],
  }

  # Configure apache vhost
  apache::vhost { $url:
    docroot             => $path,
    port                => $port,
    ssl                 => $ssl,
    ssl_cert            => $ssl_cert,
    ssl_key             => $ssl_key,
    wsgi_daemon_process => 'wsgi',
    wsgi_script_aliases => {
      '/' => "${path}/${name}/${name}/wsgi.py",
    },
    require             => Python::Virtualenv[$name],
  }

}
